@@ -1,13 +1,16 @@
+import os
+import time
+import traceback
 from datetime import datetime
 
-import time
+from elasticsearch.helpers import streaming_bulk
 from elasticsearch_dsl import Search, Q
 from flask_taxonomies.models import TaxonomyTerm
 from invenio_search import current_search_client
 
 from flask_taxonomies_es.exceptions import InvalidTermIdentification
 from flask_taxonomies_es.serializer import get_taxonomy_term
-from flask_taxonomies_es.utils import _get_taxonomy_slug_from_url
+from flask_taxonomies_es.utils import _get_taxonomy_slug_from_url, _get_tree_ids, bcolors
 
 
 class TaxonomyESAPI:
@@ -129,10 +132,11 @@ class TaxonomyESAPI:
         """
         s = Search(using=current_search_client, index=self.index)
         query = Q("match", taxonomy=taxonomy_code)
-        results = list(s.query(query))
+        search_query = s.query(query)
+        results = list(search_query.scan())
         return [result.to_dict() for result in results]
 
-    def reindex(self) -> datetime:
+    def reindex(self, taxonomies: list = None) -> datetime:
         """
         Reindex taxonomy index. Update taxonomy term and remove obsolete taxonomy terms.
 
@@ -140,23 +144,83 @@ class TaxonomyESAPI:
         :rtype: datetime
         """
         timestamp = datetime.utcnow()
-        self._synchronize_es(timestamp=timestamp)
+        self._synchronize_es(timestamp=timestamp, taxonomies=taxonomies)
         time.sleep(1)
-        self._remove_old_es_term(timestamp)
+        self._remove_old_es_term(timestamp, taxonomies=taxonomies)
         return timestamp
 
-    def _synchronize_es(self, timestamp=None) -> None:
-        with self.app.app_context():
-            for node in TaxonomyTerm.query.all():
-                self.set(node, timestamp=timestamp)
-                print(
-                    f'Taxonomy term with slug: \"{node.slug}\" from taxonomy: \"'
-                    f'{node.taxonomy.slug}\" has been updated')
+    def _synchronize_es(self, timestamp=None, taxonomies: list = None) -> None:
+        success, failed = 0, 0
+        errors = []
+        try:
+            with self.app.app_context():
+                iterator = iter(self._taxonomy_terms_generator(timestamp, taxonomies=taxonomies))
+                for ok, item in streaming_bulk(
+                        current_search_client,
+                        iterator,
+                        raise_on_exception=False):
+                    if not ok:
+                        errors.append(item)
+                        failed += 1
+                    else:
+                        success += 1
+        finally:
+            print("REINDEX RESULTS:")
+            print(f"Failed: {failed}")
+            print(f"Succeed: {success}")
+            if len(errors) > 0:
+                print("Errors:", errors)
+            print("\n\n")
 
-    def _remove_old_es_term(self, timestamp) -> None:
-        taxonomies = TaxonomyTerm.query.filter_by(level=1).all()
+    def _taxonomy_terms_generator(self, timestamp, taxonomies: list = None):
+        index_ = self.index
+        path = self.app.config["TAXONOMY_ELASTICSEARCH_LOG_DIR"]
+        if taxonomies is None:
+            query = TaxonomyTerm.query.all()
+        else:
+            tree_ids = _get_tree_ids(taxonomies)
+            query = TaxonomyTerm.query.filter(TaxonomyTerm.tree_id.in_(tree_ids))
+        if not os.path.exists(path):
+            os.mkdir(path)
+        for node in query:
+            try:
+                if node.parent:
+                    body = get_taxonomy_term(
+                        code=node.taxonomy.slug,
+                        slug=node.slug,
+                        timestamp=timestamp
+                    )
+                    yield {
+                        '_op_type': 'index',
+                        '_index': index_,
+                        '_id': node.id,
+                        '_source': body
+                    }
+            except:
+                exc_traceback = traceback.format_exc()
+                print(exc_traceback)
+                print("\n\n\n")
+                if timestamp is None:
+                    timestamp = datetime.utcnow()
+                file_name = f'{timestamp.strftime("%Y%m%dT%H%M%S")}.err'
+                file_path = os.path.join(path, file_name)
+                with open(file_path, "w") as f:
+                    f.write(
+                        f"TAXONOMY CODE: {node.taxonomy.slug}; SLUG: {node.slug}\n\n"
+                        f"{exc_traceback}")
+                continue
+
+    def _remove_old_es_term(self, timestamp, taxonomies: list = None) -> None:
+        if taxonomies is None:
+            taxonomies = TaxonomyTerm.query.filter_by(level=1).all()
+            taxonomies = [taxonomy.slug for taxonomy in taxonomies]
         for taxonomy in taxonomies:
-            for node in self.list(taxonomy.slug):
+            terms_list = self.list(taxonomy)
+            if len(terms_list) == 0:
+                print(
+                    f"{bcolors.WARNING}WARNING: Taxonomy \"{taxonomy}\" does not exist or does "
+                    f"not contain any term{bcolors.ENDC}")
+            for node in terms_list:
                 date_of_serialization = datetime.strptime(
                     node["date_of_serialization"],
                     '%Y-%m-%d %H:%M:%S.%f'
